/* Plastic sleeve for HF current probes

   A HF current probe is for EMC testing of radiated emissions from cables.
   Commercial probes are available, but expensive. This design is part of a 
   low-cost, home-made, professional grade HF-current probe.

   For details, see https://www.compuphase.com/electronics/nearfieldprobes.htm

   Copyright 2020, CompuPhase, Thiadmer Riemersma
   This work is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License
   (https://creativecommons.org/licenses/by-sa/4.0/).
 */

/* [Ferrite bead (snap-on)] */
// Outer diameter.
FerriteOuter = 25.0;
// Inner diameter (hole diameter).
FerriteInner = 15.0;
// Height (or length).
FerriteHeight = 11.5;

/* [SMA connector (panel mount)] */
// Width of the (panel mount) block
ConnectorWidth = 12.7;
// Height of the (panel mount) block
ConnectorHeight = 13.5;
// Space needed behind the connector (inside the sleeve).
ConnectorLength = 4.5;
// diameter of the hole behind the connector.
ConnectorHole = 4.5;
// Whether the "connector hole" is open on top.
ConnectorSlot = true;

/* [Wire (for windings)] */
// Diameter of the wire that is looped around one half of the ferrite. This value includes the isolation.
WireDiameter = 0.9;

/* [General sleeve parameters] */
// The thickness of top and bottom bottom covers, as well as thin walls. The recommended value depends on the 3D printer, the number of perimeters, and the nozzle.
ThinWallThickness = 0.87;
// For sturdiness, it is recommended that the outer wall is thicker.
OuterWallThickness = 1.5;
// General tolerance of all measures.
Tolerance = 0.2;
// The width of the lip used for closing the clamp.
ClampWidth = 3.0;
// The clamp is closed by pushing a clip over the connector block and the lip. This value is the width of the clip left/right and bottom sides (extra width is added to the left & right sides to avoid sharp inside corners).
ClipWidth = 3.0;
// The clamp is closed by pushing a clip over the connector block and the lip. This value is the thickness of the clip (the bigger, the stiffer and stronger).
ClipThickness = 4.0;

SleeveHole = FerriteInner - 2*WireDiameter - 2*ThinWallThickness + Tolerance;
SleeveDiameter = FerriteOuter + 2*WireDiameter + ThinWallThickness + OuterWallThickness + Tolerance;
SleeveHeight = FerriteHeight + 3*WireDiameter + 2*ThinWallThickness + Tolerance;

/* [Hinge] */
// The number of rings on the hinge should be an odd value; 5 is probably optimal.
HingeRings = 5;
// The hole in the knuckles for the pin.
HingeHole = 2.0;
// Offset of the hinge from the wall.
HingeOffset = 0.3;
// The tolerance for the hinge is separate, for getting the best fit
HingeTolerance = 0.15;
HingeWidth = SleeveHeight / HingeRings;
HingeRadius = HingeHole / 2 + 1.5;

/* [Hidden] */
epsilon = 0.01; // arbitrary small value to avoid non-manifold errors
$fn = 100;

use <hinge.scad>

module ring(outer, inner, height) {
  difference() {
    cylinder(d=outer, h=height);
    translate([0, 0, -epsilon])
      cylinder(d=inner, h=height+2*epsilon);
  }
}

module sleeve(primary) { 
  difference() {
    ring_hole = FerriteInner - Tolerance - (primary ? 2*WireDiameter : 0);
    ring_diameter = FerriteOuter + Tolerance + (primary ? 2*WireDiameter : 0);
    union() {
      // main cylinder
      cylinder(d = SleeveDiameter, h = SleeveHeight);
      
      // hinge
      if (primary) {
        translate([0, SleeveDiameter/2 + HingeRadius + HingeOffset, HingeWidth + HingeTolerance])
          rotate([0, -90, 0])
            hinge(HingeWidth - HingeTolerance, HingeWidth + HingeTolerance, floor(HingeRings/2), HingeRadius, HingeHole/2, HingeOffset+1, 4.5);
      } else {
        translate([0, SleeveDiameter/2 + HingeRadius + HingeOffset, SleeveHeight])
          rotate([0, 90, 0])
            hinge(HingeWidth - HingeTolerance, HingeWidth + HingeTolerance, ceil(HingeRings/2), HingeRadius, HingeHole/2, HingeOffset+1, 4.5);
      }
      
      // connector block (only for primary half); or rim to snap the sleeve 
      // closed (only for secondary half)
      if (primary) {
        translate([0, -SleeveDiameter/2-ConnectorLength, SleeveHeight - ConnectorHeight]) {
          difference() {
            cube([ConnectorWidth, ConnectorLength + SleeveDiameter/2, ConnectorHeight - ThinWallThickness - epsilon]);
            translate([ConnectorWidth/2, -epsilon, ConnectorHeight/2])
              rotate([-90, 0, 0])
                cylinder(d=ConnectorHole, h=ConnectorLength + SleeveDiameter/2 + 2*epsilon);
            if (ConnectorSlot)
              translate([(ConnectorWidth - ConnectorHole)/2, -epsilon, ConnectorHeight/2])
                cube([ConnectorHole, ConnectorLength + SleeveDiameter/2, ConnectorHeight/2]);
          }
        }
      } else {
        translate([-ClampWidth, -SleeveDiameter/2-ConnectorLength, SleeveHeight - ConnectorHeight])
          cube([ClampWidth, ConnectorLength + SleeveDiameter/2, ConnectorHeight]);
      }
    }

    // cut out the main hole
    translate([0, 0, -epsilon])
      cylinder(d = SleeveHole, h = SleeveHeight + 2*epsilon);
    // cut out the ring for the ferrite (plus the wiring around it, for the
    // primary half)
    translate([0, 0, ThinWallThickness + (primary ? 0 : WireDiameter)])
      ring(ring_diameter, ring_hole, SleeveHeight);

    // for non-primary half (without the windings), add another ring to help
    // align the cover (which is glued on top of the sleeve)
    translate([0, 0, SleeveHeight - ThinWallThickness])
      ring(SleeveDiameter+2*epsilon, FerriteInner - Tolerance - 2*WireDiameter, ThinWallThickness + epsilon);
    
    // for the primary half, add a slot to draw the wires through
    if (primary) {
      translate([0, -SleeveDiameter/2, SleeveHeight - 3*WireDiameter]) {
        translate([ConnectorWidth/2, 0, 0])
          rotate([-90, 0, 0])
            cylinder(d = ConnectorHole, h = (SleeveDiameter - SleeveHole)/2);
          if (ConnectorSlot)
            translate([(ConnectorWidth - ConnectorHole)/2, 0, 0])
              cube([ConnectorHole, (SleeveDiameter - SleeveHole)/2, 3*WireDiameter]);
      }
    }
    
    // cutting plane (the sleeve consists of two halves */
    offsx = primary ? -(SleeveDiameter/2 + epsilon) : 0;
    offsy = -(SleeveDiameter/2 + epsilon);
    translate([offsx, offsy, -epsilon])
      cube([SleeveDiameter/2 + epsilon, SleeveDiameter + 2*epsilon, SleeveHeight + 2*epsilon]);
  }    
}

module cover(primary) {
  difference() {
    union() {
      // main cylinder
      cylinder(d = SleeveDiameter - Tolerance, h = ThinWallThickness);
      
      // cover over the connector block (only for primary half); or extra
      // ring to fill up the space for the (non-present) windings (only for
      // secondary half)
      if (primary) {
        translate([0, -SleeveDiameter/2-ConnectorLength, 0])
          cube([ConnectorWidth, ConnectorLength + SleeveDiameter/2, ThinWallThickness]);
      } else {
        translate([0, 0, ThinWallThickness - Tolerance])
          ring(FerriteOuter - Tolerance, FerriteInner + Tolerance, 2*WireDiameter - Tolerance);      
      }
    }
    // the main hole
    translate([0, 0, -epsilon])
      cylinder(d = FerriteInner - 2*WireDiameter, h = ThinWallThickness + 2*epsilon);

    // cutting plane (the sleeve consists of two halves */
    offsx = primary ? -(SleeveDiameter/2 + epsilon) : 0;
    offsy = -(SleeveDiameter/2 + epsilon);
    translate([offsx, offsy, -epsilon])
      cube([SleeveDiameter/2 + epsilon, SleeveDiameter + 2*epsilon, ThinWallThickness + 2*WireDiameter + 2*epsilon]);
  }
}

module clip() {
  CornerRelease = 1.5;
  difference() {
    union() {
      /* base */
      translate([-(ClampWidth + ClipWidth), 0, 0])
        cube([ConnectorWidth + ClampWidth + 2*ClipWidth + 2*CornerRelease, ClipWidth, ClipThickness]);
      /* legs */
      translate([-(ClampWidth + ClipWidth), 0, 0])
        cube([ClipWidth + CornerRelease, ConnectorHeight + ClipWidth, ClipThickness]);
      translate([ConnectorWidth + CornerRelease, 0, 0])
        cube([ClipWidth + CornerRelease, ConnectorHeight + ClipWidth, ClipThickness]);
    }
    /* corner release */
    translate([-ClampWidth + CornerRelease, ClipWidth + CornerRelease, -epsilon])
      cylinder(r = CornerRelease, h = ClipThickness + 2*epsilon);
    translate([ConnectorWidth + CornerRelease, ClipWidth + CornerRelease, -epsilon])
      cylinder(r = CornerRelease, h = ClipThickness + 2*epsilon);
  }
}

translate([4, -20, 0])
  sleeve(true);
translate([-4, -20, 0])
  sleeve(false);
translate([3, 17, 0])
  mirror([0, 1, 0])
    cover(true);
translate([-3, 17, 0])
  cover(false);
translate([-4, 37, 0]) rotate([0, 0, 180])
  clip();
